package com.pokkt.thirdparty;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.Keep;

import com.adcolony.sdk.AdColony;
import com.adcolony.sdk.AdColonyAppOptions;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.pokkt.thirdparty.AdColony.AdColonyInterstitialManager;
import com.pokkt.thirdparty.AdColony.AdColonyVideoAdManager;

import java.util.Map;

@Keep
public class AdColonyNetwork implements AdNetwork {

    private static final String TAG = AdColonyNetwork.class.getName();
    private Context context = null;
    private AdNetworkInfo adNetworkInfo = null;

    /*
     * We recommend passing the AdColony client options, app ID, all zone IDs, and current zone ID
     * in the serverExtras Map by specifying Custom Network Data in Pokkt's web interface.
     *
     */
    /*
     * These keys are intended for AdColony internal use. Do not modify.
     */
    private static final String APP_ID_KEY = "appId";
    private static final String ALL_ZONE_IDS_KEY = "allZoneIds";

    private boolean isInitialized = false;
    private float amount = 0;

    private AdColonyInterstitialManager adColonyInterstitialManager;
    private AdColonyVideoAdManager adColonyVideoAdManager;

    @Override
    public void initNetwork(final Context context, final AdNetworkInfo info) {
        Logger.i(TAG + " Init network called!!");

        synchronized (AdColonyNetwork.class) {
            if (isInitialized) {
                Logger.i(TAG + " Network Already Initialized!!");
                return;
            }

            if (!(context instanceof Activity)) {
                Logger.i(TAG + " Init Configurations Error!. Needs Activity context");
                return;
            }

            if (!sdkEnabled()) {
                return;
            }

            // Set up serverExtras
            String[] adColonyAllZoneIds;
            String adColonyAppId;
            if (extrasAreValid(info)) {
                Logger.i(TAG + " Network Extras Are Valid.");
                this.context = context;
                this.adNetworkInfo = info;
                adColonyAppId = info.getCustomData().get(APP_ID_KEY);
                adColonyAllZoneIds = extractAllZoneIds(info.getCustomData());


                Logger.i(TAG + " Calling Configure !");
                // Configure ADC once early before any other ADC calls (except setCustomID/setDeviceID).
                try {
                    AdColonyAppOptions app_options = new AdColonyAppOptions().setUserID(AdManager.getInstance().getThirdPartyUserId());
                    AdColony.configure(((Activity) context), app_options, adColonyAppId, adColonyAllZoneIds);
                    Logger.i(TAG + " SDK Version: " + AdColony.getSDKVersion());
                    if (info.getCustomData().containsKey("amount")) {
                        try {
                            amount = Float.parseFloat(info.getCustomData().get("amount"));
                        } catch (Exception e) {
                            Logger.printStackTrace(TAG + " Could not parse amount", e);
                            //do nothing
                        }
                    }
                    adColonyInterstitialManager = new AdColonyInterstitialManager(info);
                    adColonyVideoAdManager = new AdColonyVideoAdManager(info);
                } catch (Throwable e) {
                    Logger.printStackTrace(TAG + " Init Configurations Error !", e);
                    return;
                }
            } else {
                Logger.i(TAG + " Network extras are not valid !");
                return;
            }
            // version - arbitrary application version
            // store   - google or amazon
            Logger.i(TAG + " Network Initialized.");
            isInitialized = true;
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    private boolean extrasAreValid(AdNetworkInfo info) {
        return info != null
                && info.getCustomData() != null
                && info.getCustomData().containsKey(APP_ID_KEY)
                && info.getCustomData().containsKey(ALL_ZONE_IDS_KEY);
    }

    private String[] extractAllZoneIds(Map<String, String> serverExtras) {
        String zoneIds = serverExtras.get(ALL_ZONE_IDS_KEY);

        return zoneIds.split(",");
    }

    @Override
    public void cacheAd(final AdConfig config, final Callbacks.WithSuccessAndFailure<Double, String> callbacks) {

        Logger.i(TAG + " Cache ad called.");
        if (!(context instanceof Activity)) {
            callbacks.onFailure("Activity Not Available. Ad Caching Failed");
            return;
        }
        if (AdFormat.VIDEO == config.adFormat) {
            adColonyVideoAdManager.fetchAd(config, callbacks);
        } else {
            adColonyInterstitialManager.fetchAd(config, callbacks);
        }
    }

    @Override
    public void showAd(final AdConfig config, final Callbacks.WithOnlyFailure<String> failureCallback) {

        Logger.i(TAG + " Play Ad called.");
        if (!(context instanceof Activity)) {
            Logger.i(TAG + " Activity Not Available ! Play ad Failed");
            failureCallback.onFailure("Activity Not Available ! Play ad Failed");
            return;
        }
        if (AdFormat.VIDEO == config.adFormat) {
            adColonyVideoAdManager.showAd(config, failureCallback);
        } else {
            adColonyInterstitialManager.showAd(config, failureCallback);
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {

        Logger.i(TAG + " CheckAdAvailability called.");
        try {
            if (AdFormat.VIDEO == config.adFormat) {
                if (adColonyVideoAdManager.checkAdAvailable(config)) {
                    callbacks.onSuccess(null);
                    return;
                }
            } else {
                if (adColonyInterstitialManager.checkAdAvailable(config)) {
                    callbacks.onSuccess(null);
                    return;
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability.", e);
        }
        callbacks.onFailure("Ad Not Available.");
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        callbacks.onFailure("Invalid Network !");
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {

    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {

        if (AdFormat.VIDEO == adConfig.adFormat) {
            adColonyVideoAdManager.notifyTimeOut(adConfig);
        } else if (AdFormat.INTERSTITIAL == adConfig.adFormat) {
            adColonyInterstitialManager.notifyTimeOut(adConfig);
        }
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        if (adConfig.isRewarded) {
            return amount;
        } else {
            return 0.0;
        }
    }

    @Override
    public boolean supportsAdConfig(AdConfig adConfig) {
        if (AdFormat.BRANDING == adConfig.adFormat) {
            return false;
        }

        if (AdFormat.INTERSTITIAL == adConfig.adFormat) {
            if (adConfig.isRewarded) {
                return false;
            }
        }
        if (AdFormat.VIDEO == adConfig.adFormat) {
            if (!(adConfig.isRewarded)) {
                return false;
            }
        }
        if (AdFormat.BANNER == adConfig.adFormat) {
            return false;
        }
        return true;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.adcolony.sdk.AdColony");
            Class.forName("com.adcolony.sdk.AdColonyInterstitial");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }
}
