package com.pokkt.thirdparty.AdMob;

import android.app.Activity;
import android.content.Context;
import android.text.TextUtils;

import com.google.android.gms.ads.AdListener;
import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.InterstitialAd;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;

import org.json.JSONObject;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class AdMobInterstitialManager {

    private static final String TAG = AdMobInterstitialManager.class.getName();

    private AdNetworkInfo adNetworkInfo;
    private Map<String, InterstitialAd> adMap = new HashMap<String, InterstitialAd>();//map of ad unit to ad
    private String testDeviceId = "";
    private AdConfig adMobAdConfig = null;
    private boolean isTimedOut;
    private Callbacks.WithSuccessAndFailure<Double, String> callback;

    public AdMobInterstitialManager(AdNetworkInfo info, String testDeviceId) {
        this.adNetworkInfo = info;
        this.testDeviceId = testDeviceId;
    }

    public void fetchAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks, Context context) {
        Logger.d(TAG + " cache Ad called !");

        final AdMobAdListener listener = new AdMobAdListener(config);

        try {
            String adUnitId = getAdUnitId(config);
            if (TextUtils.isEmpty(adUnitId)) {
                callbacks.onFailure("Ad Caching Failed");
                return;
            }
            if (this.adMobAdConfig != null && !(config.equals(this.adMobAdConfig))) {
                callbacks.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.adMobAdConfig == null) {
                this.adMobAdConfig = config.clone();
            }
            InterstitialAd ad = adMap.get(getAdUnitId(config));
            if (ad != null && ad.isLoaded()) {
                callbacks.onSuccess(0.0);
                return;
            }
            this.callback = callbacks;

            final InterstitialAd interstitialAd = new InterstitialAd(context);
            interstitialAd.setAdUnitId(adUnitId);

            listener.setAdUnitID(adUnitId);
            listener.setInterstitialAd(interstitialAd);

            //This is only used for testing "AdMob" on real device.
            ((Activity) context).runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    AdRequest.Builder requestBuilder = new AdRequest.Builder();
                    if (testDeviceId != null && testDeviceId.length() > 0) {
                        List<String> testDeviceIdList = Arrays.asList(testDeviceId.split(","));

                        for (String deviceId : testDeviceIdList) {
                            Logger.d(TAG + " Test Device Id for AdMob: " + deviceId);
                            requestBuilder.addTestDevice(deviceId);
                        }
                        AdRequest adRequest = requestBuilder.build();
                        interstitialAd.loadAd(adRequest);
                    } else {
                        interstitialAd.loadAd(new AdRequest.Builder().build());
                    }

                    interstitialAd.setAdListener(listener);
                }
            });
            isTimedOut = false;
        } catch (Throwable e) {
            this.adMobAdConfig = null;
            isTimedOut = true;
            Logger.printStackTrace(TAG + " Cache Ad failed", e);
            callbacks.onFailure("Ad Caching Failed");
        }
    }

    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {

        Logger.d(TAG + " show Interstitial called !");
        String adUnitId = "";
        try {
            adUnitId = getAdUnitId(config);
            if (TextUtils.isEmpty(adUnitId)) {
                adMap.remove(adUnitId);
                requestFailed("Ad Not Available !", failureCallback);
                return;
            }
            if (this.adMobAdConfig != null && !(config.equals(this.adMobAdConfig))) {
                failureCallback.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.adMobAdConfig == null) {
                this.adMobAdConfig = config.clone();
            }
            InterstitialAd interstitialAd = adMap.get(adUnitId);
            if (interstitialAd != null && interstitialAd.isLoaded()) {
                interstitialAd.show();
            } else {
                Logger.d(TAG + " No Ad is currently available !");
                adMap.remove(adUnitId);
                requestFailed("Ad Not Available !", failureCallback);
            }
        } catch (Throwable e) {
            adMap.remove(adUnitId);
            requestFailed("Ad Not Available !", failureCallback);
        }
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        this.adMobAdConfig = null;
        failureCallback.onFailure(msg);
    }

    public boolean isInterstitialAvailable(AdConfig config) {
        try {
            String zoneId = getAdUnitId(config);
            if (!TextUtils.isEmpty(zoneId)) {
                if (this.adMobAdConfig != null && !(config.equals(this.adMobAdConfig))) {
                    Logger.d(TAG + " Fetch Ad Failed. Wrong AdConfig.");
                    return false;
                }
            }
            InterstitialAd interstitialAd = adMap.get(getAdUnitId(config));
            if (interstitialAd != null && interstitialAd.isLoaded()) {
                Logger.d(TAG + " ad is available and ready !");
                return true;
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }

    private String getAdUnitId(AdConfig config) {
        String adUnitId = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(config.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isInterstitial = mapObj.optInt("non_incent");
                    if (isInterstitial == 1) {
                        adUnitId = mapObj.optString("network_screen", "").trim();
                    } else {
                        Logger.d(TAG + " Failed.Requesting rewarded interstitial from admob.");
                        return adUnitId;
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        if (adUnitId.length() == 0) {
            adUnitId = config.screenName;
        }
        return adUnitId;
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Ad";
        adMobAdConfig = null;
        Logger.e(TAG + reason);
    }

    private class AdMobAdListener extends AdListener {
        private AdConfig adConfig;
        private String adUnitID = "";
        private InterstitialAd interstitialAd = null;

        void setAdUnitID(String adUnitId) {
            this.adUnitID = adUnitId;
        }

        void setInterstitialAd(InterstitialAd interstitialAd) {
            this.interstitialAd = interstitialAd;
        }

        AdMobAdListener(AdConfig config) {
            this.adConfig = config;
        }

        @Override
        public void onAdFailedToLoad(int errorCode) {
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onFailure("Ad Load Failed ! " + errorCode);
                adMobAdConfig = null;
            }
        }

        @Override
        public void onAdLoaded() {
            Logger.d(TAG + " onAdLoadSucceeded !");
            adMap.put(adUnitID, interstitialAd);

            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(0.0);
            }

        }

        @Override
        public void onAdLeftApplication() {

        }

        @Override
        public void onAdOpened() {
            Logger.d(TAG + " onAdOpened !");
            AdManager.getInstance().adDisplayed(adConfig, adNetworkInfo);
        }

        @Override
        public void onAdClosed() {
            Logger.d(TAG + " onAdClosed !");
            adMap.remove(adUnitID);
            AdManager.getInstance().adCompleted(adConfig, adNetworkInfo);
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            adMobAdConfig = null;
        }
    }
}