package com.pokkt.thirdparty;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.Keep;

import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.pokkt.thirdparty.AdMob.AdMobBannerManager;
import com.pokkt.thirdparty.AdMob.AdMobInterstitialManager;
import com.pokkt.thirdparty.AdMob.AdMobVideoAdManager;

@Keep
public class AdMobNetwork implements AdNetwork {

    private static final String TAG = AdMobNetwork.class.getName();
    private static final String TEST_DEVICE_ID = "TestDeviceId";

    private Context context = null;
    private double amount = 0;
    private AdNetworkInfo adNetworkInfo;
    private boolean isInitialized = false;
    private AdMobVideoAdManager adMobVideoAdManager;
    private AdMobInterstitialManager adMobInterstitialManager;
    private AdMobBannerManager adMobBannerManager;

    @Override
    public void initNetwork(final Context context, final AdNetworkInfo networkInfo) {
        Logger.d(TAG + " Init network called !");

        synchronized (AdMobNetwork.class) {
            if (isInitialized) {
                Logger.d(TAG + " Network already initialized !");
                return;
            }
            if (!(context instanceof Activity)) {
                Logger.d(TAG + " Activity context needed !");
                return;
            }

            if (!sdkEnabled()) {
                Logger.d(TAG + " Init Configurations Error !");
                return;
            }

            this.context = context;

            if (extrasAreValid(networkInfo)) {
                this.adNetworkInfo = networkInfo;
                String testDeviceId = networkInfo.getCustomData().get(TEST_DEVICE_ID);
                Logger.d(TAG + " init network initialized !");

                if (networkInfo.getCustomData().containsKey("amount")) {
                    try {
                        amount = Float.parseFloat(networkInfo.getCustomData().get("amount"));
                    } catch (Exception e) {
                        Logger.printStackTrace(TAG + " Could not parse amount", e);
                        //do nothing
                    }
                }
                adMobVideoAdManager = new AdMobVideoAdManager(networkInfo, testDeviceId);
                adMobInterstitialManager = new AdMobInterstitialManager(networkInfo, testDeviceId);
                adMobBannerManager = new AdMobBannerManager(networkInfo, testDeviceId);
                isInitialized = true;
            } else {
                Logger.d(TAG + " Init Configurations Error or SDK Error !");
            }
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        if (AdFormat.VIDEO == config.adFormat) {
            adMobVideoAdManager.fetchAd(config, callbacks, context);
        } else {
            adMobInterstitialManager.fetchAd(config, callbacks, context);
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        if (AdFormat.VIDEO == config.adFormat) {
            adMobVideoAdManager.showAd(config, failureCallback);
        } else {
            adMobInterstitialManager.showAd(config, failureCallback);
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (AdFormat.VIDEO == config.adFormat) {
                if (adMobVideoAdManager.isVideoAvailable(config)) {
                    callbacks.onSuccess(null);
                    return;
                }
            } else {
                if (adMobInterstitialManager.isInterstitialAvailable(config)) {
                    callbacks.onSuccess(null);
                    return;
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        callbacks.onFailure(null);
    }

    @Override
    public void loadBannerAd(final AdConfig adConfig, final BannerUnit bannerUnit, final Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            ((Activity)context).runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    adMobBannerManager.loadBanner(adConfig, bannerUnit, callbacks, context);
                }
            });
        } catch (Throwable e) {
            callbacks.onFailure(" load banner Failed");
        }
    }

    private boolean extrasAreValid(AdNetworkInfo info) {
        return info != null && info.getCustomData() != null;
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {
        adMobBannerManager.destroyBanner(adConfig);
    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {

        if (AdFormat.VIDEO == adConfig.adFormat) {
            adMobVideoAdManager.notifyCachingTimeout();
        } else if (AdFormat.INTERSTITIAL == adConfig.adFormat) {
            adMobInterstitialManager.notifyCachingTimeout();
        } else if (AdFormat.BANNER == adConfig.adFormat) {
            adMobBannerManager.notifyCachingTimeout();
        }
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        if (adConfig.isRewarded) {
            return amount;
        } else {
            return 0.0;
        }
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        if (AdFormat.BRANDING == config.adFormat) {
            return false;
        }

        if (AdFormat.INTERSTITIAL == config.adFormat) {
            if (config.isRewarded) {
                return false;
            }
        }
        if (AdFormat.VIDEO == config.adFormat) {
            if (!(config.isRewarded)) {
                return false;
            }
        }
        return true;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.google.android.gms.ads.AdRequest");
            Class.forName("com.google.android.gms.ads.AdListener");
            Class.forName("com.google.android.gms.ads.InterstitialAd");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }
}