package com.pokkt.thirdparty.CustomAdMob;

import android.content.Context;
import android.text.TextUtils;

import com.google.android.gms.ads.AdListener;
import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.AdSize;
import com.google.android.gms.ads.AdView;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.banners.PokktBannerView;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;

import org.json.JSONObject;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CustomAdMobBannerManager {

    private static final String TAG = CustomAdMobBannerManager.class.getName();

    private AdNetworkInfo adNetworkInfo;
    private String testDeviceId = "";
    private boolean isTimedOut;
    private Callbacks.WithSuccessAndFailure<AdCampaign, String> callback;
    private Map<AdConfig, AdView> adMap = new HashMap<AdConfig, AdView>();//map of screenname to ad

    public CustomAdMobBannerManager(AdNetworkInfo info, String testDeviceId) {
        this.adNetworkInfo = info;
        this.testDeviceId = testDeviceId;
    }

    public void loadBanner(AdConfig config, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks, Context context) {
        Logger.d(TAG + " cache Ad called !");

        AdMobAdListener listener = new AdMobAdListener(config);

        try {
            String adUnitId = getAdUnitId(config);
            if (TextUtils.isEmpty(adUnitId)) {
                callbacks.onFailure("Invalid Ad Unit");
                return;
            }
            this.callback = callbacks;
            AdRequest adRequest = null;

            AdView adUnit = adMap.get(config);
            if (adUnit != null) {
                callbacks.onSuccess(null);
                return;
            }
            AdView adview = new AdView(context);
            adview.setAdUnitId(adUnitId);
            AdSize size = getBannerSize(config);
            adview.setAdSize(size);
            listener.setbannerAd(adview);
            listener.setBannerAdView(bannerUnit);

            //This is only used for testing "AdMob" on real device.
            AdRequest.Builder requestBuilder = new AdRequest.Builder();
            if (testDeviceId != null && testDeviceId.length() > 0) {
                List<String> testDeviceIdList = Arrays.asList(testDeviceId.split(","));

                for (String deviceId : testDeviceIdList) {
                    Logger.d(TAG + " Test Device Id for AdMob: " + deviceId);
                    requestBuilder.addTestDevice(deviceId);
                }
                adRequest = requestBuilder.build();
                adview.loadAd(adRequest);
            } else {
                adview.loadAd(new AdRequest.Builder().build());
            }

            isTimedOut = false;
            adview.setAdListener(listener);
        } catch (Throwable e) {
            isTimedOut = true;
            Logger.printStackTrace(TAG + " Cache Ad failed", e);
            callbacks.onFailure(" Ad Caching Failed");
        }
    }

    private AdSize getBannerSize(AdConfig view) {
        int height = view.sHeight;
        int width = view.sWidth;
        if (height >= AdSize.BANNER.getHeight() && width >= AdSize.BANNER.getWidth())
            return AdSize.BANNER;
        if (height >= AdSize.FULL_BANNER.getHeight() && width >= AdSize.FULL_BANNER.getWidth())
            return AdSize.FULL_BANNER;
        if (height >= AdSize.LARGE_BANNER.getHeight() && width >= AdSize.LARGE_BANNER.getWidth())
            return AdSize.LARGE_BANNER;
        if (height >= AdSize.LEADERBOARD.getHeight() && width >= AdSize.LEADERBOARD.getWidth())
            return AdSize.LEADERBOARD;
        if (height >= AdSize.MEDIUM_RECTANGLE.getHeight() && width >= AdSize.MEDIUM_RECTANGLE.getWidth())
            return AdSize.MEDIUM_RECTANGLE;
        if (height >= AdSize.WIDE_SKYSCRAPER.getHeight() && width >= AdSize.WIDE_SKYSCRAPER.getWidth())
            return AdSize.WIDE_SKYSCRAPER;
        return AdSize.SMART_BANNER;
    }

    private String getAdUnitId(AdConfig config) {
        String adUnitId = "";
        if (adNetworkInfo.getCustomData().containsKey("extra_param")) {
            String extraParam = adNetworkInfo.getCustomData().get("extra_param");
            if (!TextUtils.isEmpty(extraParam)) {
                try {
                    JSONObject extraParamObj = new JSONObject(extraParam);
                    String screenMapping = extraParamObj.optString("screens_mapping_data");
                    JSONObject mappingObject = new JSONObject(screenMapping);
                    String mapData = mappingObject.optString(config.screenName);
                    if (!TextUtils.isEmpty(mapData)) {
                        JSONObject mapObj = new JSONObject(mapData);
                        int isBanner = mapObj.optInt("non_incent");
                        if (isBanner == 1) {
                            adUnitId = mapObj.optString("network_screen", "").trim();
                        } else {
                            Logger.d(TAG + " Failed.Requesting rewarded banner from admob.");
                            return adUnitId;
                        }
                    }
                } catch (Exception e) {
                    Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
                }
            }
        }
        if (adUnitId.length() == 0) {
            adUnitId = config.screenName;
        }
        return adUnitId;
    }

    public void destroyBanner(AdConfig adConfig) {
        String adUnitId = getAdUnitId(adConfig);
        if (!(TextUtils.isEmpty(adUnitId))) {
            AdView unit = adMap.get(adConfig);
            if (unit != null) {
                unit.destroy();
                adMap.remove(adConfig);
            }
        }
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Ad";
        Logger.e(TAG + reason);
    }

    private class AdMobAdListener extends AdListener {
        private final AdConfig adConfig;
        private AdView adView = null;
        private BannerUnit bannerUnit;

        AdMobAdListener(AdConfig config) {
            this.adConfig = config;
        }

        void setbannerAd(AdView adView) {
            this.adView = adView;
        }

        @Override
        public void onAdFailedToLoad(int errorCode) {
            if (!isTimedOut) {
                Logger.d(TAG + " banner load failed " + errorCode);
                isTimedOut = true;
                callback.onFailure("Ad Load Failed " + errorCode);
                if (adView != null) {
                    adView.destroy();
                }
            }
        }

        @Override
        public void onAdLoaded() {
            Logger.d(TAG + " onAdLoadSucceeded !");

            if (!isTimedOut) {
                if (!(adMap.containsKey(adConfig))) {
                    adMap.put(adConfig, adView);
                }
                isTimedOut = true;
                List<PokktBannerView> containerList = bannerUnit.getContainerList();
//                for (int i = 0; i < containerList.size(); i++) {
                if (containerList.size() > 0) {
                    PokktBannerView container = containerList.get(0);
                    if (container.getChildAt(0) == null) {
                        container.addView(adView);
                    } else {
                        container.removeView(container.getChildAt(0));
                        container.addView(adView);
                    }
                    callback.onSuccess(null);
                } else {
                    destroyBanner(adConfig);
                }
//                }
            }

        }

        @Override
        public void onAdLeftApplication() {
            if (adView != null) {
                Logger.d(TAG + " banner leave app ");
                adView.destroy();
            }
        }

        @Override
        public void onAdOpened() {
            Logger.d(TAG + " onAdOpened !");
        }

        @Override
        public void onAdClosed() {
            Logger.d(TAG + " onAdClosed !");
        }

        void setBannerAdView(BannerUnit unit) {
            this.bannerUnit = unit;
        }
    }
}