package com.pokkt.thirdparty.CustomAdMob;

import android.content.Context;
import android.text.TextUtils;

import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.MobileAds;
import com.google.android.gms.ads.reward.RewardItem;
import com.google.android.gms.ads.reward.RewardedVideoAd;
import com.google.android.gms.ads.reward.RewardedVideoAdListener;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;

import org.json.JSONObject;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CustomAdMobVideoAdManager {

    private static final String TAG = CustomAdMobVideoAdManager.class.getName();
    private AdNetworkInfo adNetworkInfo;
    private Map<String, RewardedVideoAd> adMap = new HashMap<>();//map of ad unit to ad
    private String testDeviceId = "";
    private AdConfig adMobAdConfig = null;
    private Callbacks.WithSuccessAndFailure<Double, String> callback;
    private boolean isTimedOut;

    public CustomAdMobVideoAdManager(AdNetworkInfo info, String testDeviceId) {
        this.adNetworkInfo = info;
        this.testDeviceId = testDeviceId;
    }

    public void fetchAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks, Context context) {
        Logger.d(TAG + " cache Ad called !");

        AdMobAdListener listener = new AdMobAdListener(config);

        try {
            String adUnitId = getAdUnitId(config);
            if (TextUtils.isEmpty(adUnitId)) {
                callbacks.onFailure("Ad Caching Failed");
                return;
            }
            if (this.adMobAdConfig != null && !(config.equals(this.adMobAdConfig))) {
                callbacks.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.adMobAdConfig == null) {
                this.adMobAdConfig = config.clone();
            }
            RewardedVideoAd ad = adMap.get(getAdUnitId(config));
            if (ad != null && ad.isLoaded()) {
                callbacks.onSuccess(0.0);
                return;
            }
            AdRequest adRequest = null;
            this.callback = callbacks;

            RewardedVideoAd videoAd = MobileAds.getRewardedVideoAdInstance(context);

            listener.setAdUnitID(adUnitId);
            listener.setInterstitialAd(videoAd);

            //This is only used for testing "AdMob" on real device.
            AdRequest.Builder requestBuilder = new AdRequest.Builder();
            if (testDeviceId != null && testDeviceId.length() > 0) {
                List<String> testDeviceIdList = Arrays.asList(testDeviceId.split(","));

                for (String deviceId : testDeviceIdList) {
                    Logger.d(TAG + " Test Device Id for AdMob: " + deviceId);
                    requestBuilder.addTestDevice(deviceId);
                }
                adRequest = requestBuilder.build();
                videoAd.loadAd(adUnitId, adRequest);
            } else {
                videoAd.loadAd(adUnitId, new AdRequest.Builder().build());
            }
            isTimedOut = false;

            videoAd.setRewardedVideoAdListener(listener);
        } catch (Throwable e) {
            this.adMobAdConfig = null;
            isTimedOut = true;
            Logger.printStackTrace(TAG + " Cache Ad Failed ", e);
            callbacks.onFailure("Ad Caching Failed");
        }
    }

    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {

        Logger.d(TAG + " show Interstitial called !");
        String adUnitId = "";
        try {
            adUnitId = getAdUnitId(config);
            if (TextUtils.isEmpty(adUnitId)) {
                adMap.remove(adUnitId);
                requestFailed("Ad Not Available !", failureCallback);
                return;
            }
            if (this.adMobAdConfig != null && !(config.equals(this.adMobAdConfig))) {
                failureCallback.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.adMobAdConfig == null) {
                this.adMobAdConfig = config.clone();
            }
            RewardedVideoAd interstitialAd = adMap.get(adUnitId);
            if (interstitialAd != null && interstitialAd.isLoaded()) {
                interstitialAd.show();
            } else {
                Logger.d(TAG + " No Ad is currently available!!");
                adMap.remove(adUnitId);
                requestFailed("Ad Not Available !", failureCallback);
            }
        } catch (Throwable e) {
            adMap.remove(adUnitId);
            requestFailed("Ad Not Available !", failureCallback);
        }
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        this.adMobAdConfig = null;
        failureCallback.onFailure(msg);
    }

    public boolean isVideoAvailable(AdConfig config) {
        try {
            String zoneId = getAdUnitId(config);
            if (!TextUtils.isEmpty(zoneId)) {
                if (this.adMobAdConfig != null && !(config.equals(this.adMobAdConfig))) {
                    Logger.d(TAG + " Fetch Ad Failed. Wrong AdConfig.");
                    return false;
                }
            }
            RewardedVideoAd ad = adMap.get(getAdUnitId(config));
            if (ad != null && ad.isLoaded()) {
                Logger.d(TAG + " ad is available and ready !!");
                return true;
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }

    private String getAdUnitId(AdConfig config) {
        String adUnitId = "";
        if (adNetworkInfo.getCustomData().containsKey("extra_param")) {
            String extraParam = adNetworkInfo.getCustomData().get("extra_param");
            if (!TextUtils.isEmpty(extraParam)) {
                try {
                    JSONObject extraParamObj = new JSONObject(extraParam);
                    String screenMapping = extraParamObj.optString("screens_mapping_data");
                    JSONObject mappingObject = new JSONObject(screenMapping);
                    String mapData = mappingObject.optString(config.screenName);
                    if (!TextUtils.isEmpty(mapData)) {
                        JSONObject mapObj = new JSONObject(mapData);
                        int isVideo = mapObj.optInt("incent");
                        if (isVideo == 1) {
                            adUnitId = mapObj.optString("network_screen", "").trim();
                        } else {
                            Logger.d(TAG + " Failed.Requesting non rewarded video from admob.");
                            return adUnitId;
                        }
                    }
                } catch (Exception e) {
                    Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
                }
            }
        }
        if (adUnitId.length() == 0) {
            adUnitId = config.screenName;
        }
        return adUnitId;
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Ad";
        adMobAdConfig = null;
        Logger.e(TAG + reason);
    }

    private class AdMobAdListener implements RewardedVideoAdListener {
        private final AdConfig adConfig;
        private String adUnitID = "";
        private RewardedVideoAd videoAd = null;

        void setAdUnitID(String adUnitId) {
            this.adUnitID = adUnitId;
        }

        void setInterstitialAd(RewardedVideoAd interstitialAd) {
            this.videoAd = interstitialAd;
        }

        AdMobAdListener(AdConfig config) {
            this.adConfig = config;
        }

        @Override
        public void onRewardedVideoAdLoaded() {
            Logger.d(TAG + " onAdLoadSucceeded !!");
            adMap.put(adUnitID, videoAd);

            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(0.0);
            }
        }

        @Override
        public void onRewardedVideoAdOpened() {
            Logger.d(TAG + " onAdOpened !");
            AdManager.getInstance().adDisplayed(adConfig, adNetworkInfo);
        }

        @Override
        public void onRewardedVideoStarted() {

        }

        @Override
        public void onRewardedVideoAdClosed() {
            Logger.d(TAG + " onAdClosed !");
            adMap.remove(adUnitID);
            adMobAdConfig = null;
            isTimedOut = true;
            AdManager.getInstance().adCompleted(adConfig, adNetworkInfo);
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
        }

        @Override
        public void onRewarded(RewardItem rewardItem) {
            if (adConfig.isRewarded) {
                AdManager.getInstance().adGratified(adConfig, rewardItem.getAmount(), adNetworkInfo);
            }
        }

        @Override
        public void onRewardedVideoAdLeftApplication() {

        }

        @Override
        public void onRewardedVideoAdFailedToLoad(int errorCode) {
            if (!isTimedOut) {
                adMobAdConfig = null;
                isTimedOut = true;
                callback.onFailure("Ad Load Failed ! " + errorCode);
            }
        }
    }
}