package com.pokkt.thirdparty;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.Keep;
import android.text.TextUtils;

import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.pokkt.thirdparty.CustomAdMob.CustomAdMobBannerManager;
import com.pokkt.thirdparty.CustomAdMob.CustomAdMobInterstitialManager;
import com.pokkt.thirdparty.CustomAdMob.CustomAdMobVideoAdManager;

import org.json.JSONObject;

@Keep
public class CustomAdMobNetwork implements AdNetwork {

    private static final String TAG = CustomAdMobNetwork.class.getName();
    private static final String TEST_DEVICE_ID = "TestDeviceId";

    private Context context = null;
    private double amount = 0;
    private AdNetworkInfo adNetworkInfo;
    private boolean isInitialized = false;
    private CustomAdMobVideoAdManager adMobVideoAdManager;
    private CustomAdMobInterstitialManager adMobInterstitialManager;
    private CustomAdMobBannerManager adMobBannerManager;

    @Override
    public void initNetwork(final Context context, final AdNetworkInfo networkInfo) {
        Logger.d(TAG + " Init network called !");

        synchronized (CustomAdMobNetwork.class) {
            if (isInitialized) {
                Logger.d(TAG + " Network already initialized !");
                return;
            }
            if (!(context instanceof Activity)) {
                Logger.d(TAG + " Activity context needed !");
                return;
            }

            if (!sdkEnabled()) {
                Logger.d(TAG + " Init Configurations Error !");
                return;
            }

            this.context = context;

            if (extrasAreValid(networkInfo)) {
                this.adNetworkInfo = networkInfo;
                String testDeviceId = "";
                if (networkInfo.getCustomData().containsKey("extra_param")) {
                    String extraParam = networkInfo.getCustomData().get("extra_param");
                    if (!TextUtils.isEmpty(extraParam)) {
                        try {
                            JSONObject extraParamObj = new JSONObject(extraParam);
                            testDeviceId = extraParamObj.optString(TEST_DEVICE_ID);
                            amount = extraParamObj.optDouble("amount", 0);
                        } catch (Throwable e) {
                            Logger.printStackTrace(TAG + " Could not parse extra param", e);
                        }
                    }
                    Logger.d(TAG + " init network initialized !");
                }
                adMobVideoAdManager = new CustomAdMobVideoAdManager(networkInfo, testDeviceId);
                adMobInterstitialManager = new CustomAdMobInterstitialManager(networkInfo, testDeviceId);
                adMobBannerManager = new CustomAdMobBannerManager(networkInfo, testDeviceId);
                isInitialized = true;
            } else {
                Logger.d(TAG + " Init Configurations Error or SDK Error!");
            }
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        if (AdFormat.VIDEO == config.adFormat) {
            adMobVideoAdManager.fetchAd(config, callbacks, context);
        } else {
            adMobInterstitialManager.fetchAd(config, callbacks, context);
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        if (AdFormat.VIDEO == config.adFormat) {
            adMobVideoAdManager.showAd(config, failureCallback);
        } else {
            adMobInterstitialManager.showAd(config, failureCallback);
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (AdFormat.VIDEO == config.adFormat) {
                if (adMobVideoAdManager.isVideoAvailable(config)) {
                    callbacks.onSuccess(null);
                    return;
                }
            } else {
                if (adMobInterstitialManager.isInterstitialAvailable(config)) {
                    callbacks.onSuccess(null);
                    return;
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        callbacks.onFailure(null);
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        adMobBannerManager.loadBanner(adConfig, bannerUnit, callbacks, context);
    }

    private boolean extrasAreValid(AdNetworkInfo info) {
        return info != null && info.getCustomData() != null;
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {
        adMobBannerManager.destroyBanner(adConfig);
    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {

        if (AdFormat.VIDEO == adConfig.adFormat) {
            adMobVideoAdManager.notifyCachingTimeout();
        } else if (AdFormat.INTERSTITIAL == adConfig.adFormat) {
            adMobInterstitialManager.notifyCachingTimeout();
        } else if (AdFormat.BANNER == adConfig.adFormat) {
            adMobBannerManager.notifyCachingTimeout();
        }
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        if (adConfig.isRewarded) {
            return amount;
        } else {
            return 0.0;
        }
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        if (AdFormat.BRANDING == config.adFormat) {
            return false;
        }

        if (AdFormat.INTERSTITIAL == config.adFormat) {
            if (config.isRewarded) {
                return false;
            }
        }
        if (AdFormat.VIDEO == config.adFormat) {
            if (!(config.isRewarded)) {
                return false;
            }
        }
        return true;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.google.android.gms.ads.AdRequest");
            Class.forName("com.google.android.gms.ads.AdListener");
            Class.forName("com.google.android.gms.ads.InterstitialAd");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }
}