package com.pokkt.thirdparty.Facebook;

import android.app.Activity;
import android.content.Context;
import android.text.TextUtils;

import com.facebook.ads.Ad;
import com.facebook.ads.AdError;
import com.facebook.ads.AdSettings;
import com.facebook.ads.InterstitialAd;
import com.facebook.ads.InterstitialAdListener;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;

import org.json.JSONObject;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class FacebookInterstitialManager {

    private static final String TAG = FacebookInterstitialManager.class.getName();

    private AdNetworkInfo adNetworkInfo;
    private Map<AdConfig, InterstitialAd> adMap = new HashMap<>();
    private String testDeviceId = "";
    private AdConfig facebookAdConfig;
    private Callbacks.WithSuccessAndFailure<Double, String> callback;
    private boolean isTimedOut;

    public FacebookInterstitialManager(AdNetworkInfo network, String testDeviceId) {
        this.adNetworkInfo = network;
        this.testDeviceId = testDeviceId;
    }

    public void fetchAd(final AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks, Context context) {
        Logger.d(TAG + " cache Ad called !");

        FacebookAdListener listener = new FacebookAdListener(config);
        try {
            if (this.facebookAdConfig != null && !(config.equals(this.facebookAdConfig))) {
                callbacks.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.facebookAdConfig == null) {
                this.facebookAdConfig = config.clone();
            }
            String placementId = getPlacementId(config);
            if (TextUtils.isEmpty(placementId)) {
                callbacks.onFailure("Ad Caching Failed");
                return;
            }
            InterstitialAd ad = adMap.get(config);
            if (ad != null && ad.isAdLoaded()) {
                callbacks.onSuccess(0.0);
                return;
            }

            this.callback = callbacks;

            final InterstitialAd interstitialAd = new InterstitialAd(context, placementId);
            interstitialAd.setAdListener(listener);

            listener.setInterstitialAd(interstitialAd);

            //This is only used for testing "Facebook Ad" on real device.
            ((Activity) context).runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    if (testDeviceId != null && testDeviceId.length() > 0) {
                        List<String> testDeviceIdList = Arrays.asList(testDeviceId.split(","));

                        for (String deviceId : testDeviceIdList)
                            Logger.d(TAG + " Test Device Id for Facebook Ad: " + deviceId);

                        AdSettings.addTestDevices(testDeviceIdList);
                    }
                    interstitialAd.loadAd();
                }
            });
            isTimedOut = false;

        } catch (Throwable e) {
            facebookAdConfig = null;
            isTimedOut = true;
            Logger.printStackTrace(TAG + " Cache Ad failed", e);
            callbacks.onFailure("Cache Ad Failed");
        }
    }

    public void showAd(final AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.d(TAG + " Show interstitial called !");

        String placementId = getPlacementId(config);
        try {
            if (TextUtils.isEmpty(placementId)) {
                adMap.remove(config);
                requestFailed("interstitial Not Available !", failureCallback);
                return;
            }
            if (this.facebookAdConfig != null && !(config.equals(this.facebookAdConfig))) {
                failureCallback.onFailure("Fetch interstitial Failed. Wrong AdConfig.");
                return;
            }
            if (this.facebookAdConfig == null) {
                this.facebookAdConfig = config.clone();
            }
            InterstitialAd interstitialAd = adMap.get(config);
            if (interstitialAd != null && interstitialAd.isAdLoaded()) {
                interstitialAd.show();
            } else {
                Logger.d(TAG + " No interstitial is currently available !");
                adMap.remove(config);
                requestFailed("Show interstitial failed", failureCallback);
            }
        } catch (Throwable e) {
            adMap.remove(config);
            requestFailed("Show interstitial failed", failureCallback);
        }
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        facebookAdConfig = null;
        failureCallback.onFailure(msg);
    }

    public boolean isInterstitialAvailable(AdConfig config) {
        try {
            String placementId = getPlacementId(config);
            if (!TextUtils.isEmpty(placementId)) {
                if (this.facebookAdConfig != null && !(config.equals(this.facebookAdConfig))) {
                    Logger.d(TAG + " Fetch Ad Failed. Wrong AdConfig.");
                    return false;
                }
            }
            InterstitialAd interstitialAd = adMap.get(config);
            if (interstitialAd != null && interstitialAd.isAdLoaded()) {
                Logger.d(TAG + " ad is available and ready !!");
                return true;
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }

    private String getPlacementId(AdConfig config) {
        String placementId = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(config.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isInterstitial = mapObj.optInt("non_incent");
                    if (isInterstitial == 1) {
                        placementId = mapObj.optString("network_screen", "").trim();
                    } else {
                        Logger.d(TAG + " Failed.Requesting rewarded interstitial from facebook.");
                        return placementId;
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        if (placementId.length() == 0) {
            placementId = config.screenName;
        }
        return placementId;
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching Interstitial";
        Logger.e(TAG + reason);
        facebookAdConfig = null;
    }

    private class FacebookAdListener implements InterstitialAdListener {

        private final AdConfig adConfig;
        private InterstitialAd interstitialAd = null;

        void setInterstitialAd(InterstitialAd interstitialAd) {
            this.interstitialAd = interstitialAd;
        }

        FacebookAdListener(AdConfig config) {
            this.adConfig = config;
        }

        @Override
        public void onError(Ad ad, AdError adError) {
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onFailure(adError.getErrorMessage());
                facebookAdConfig = null;
            }
        }

        @Override
        public void onAdLoaded(Ad ad) {
            Logger.d(TAG + " onAdLoadSucceeded !");
            adMap.put(adConfig, interstitialAd);

            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(0.0);
            }
        }

        @Override
        public void onAdClicked(Ad ad) {

        }

        @Override
        public void onLoggingImpression(Ad ad) {

        }

        @Override
        public void onInterstitialDisplayed(Ad ad) {
            Logger.d(TAG + " onInterstitialDisplayed !");
            AdManager.getInstance().adDisplayed(adConfig, adNetworkInfo);
        }

        @Override
        public void onInterstitialDismissed(Ad ad) {
            Logger.d(TAG + " onAdDismissed !");
            AdManager.getInstance().adCompleted(adConfig, adNetworkInfo);
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            adMap.remove(adConfig);
            facebookAdConfig = null;
        }
    }
}
