package com.pokkt.thirdparty;

import android.content.Context;
import android.support.annotation.Keep;

import com.apptracker.android.listener.AppModuleListener;
import com.apptracker.android.track.AppTracker;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;

@Keep
public class LeadboltNetwork implements AdNetwork {

    private static final String TAG = LeadboltNetwork.class.getName();
    private boolean isInitialized = false;
    private AdNetworkInfo adNetworkInfo;
    private AdConfig adConfig;
    private Context context;
    private Callbacks.WithSuccessAndFailure<Double, String> callback;
    private boolean isTimedOut = false;
    private AdListener listener;
    //Leadbolt can have separate video and interstitial

    @Override
    public void initNetwork(final Context context, AdNetworkInfo info) {
        Logger.i(TAG + " Init network called!!");

        synchronized (LeadboltNetwork.class) {
            if (isInitialized) {
                Logger.i(TAG + " Network already initialized !");
                return;
            }

            if (!sdkEnabled()) {
                return;
            }

            this.adNetworkInfo = info;
            this.context = context;
            if (adNetworkInfo != null && adNetworkInfo.getCustomData() != null && adNetworkInfo.getCustomData().containsKey("ApiKey")) {
                final String apiKey = adNetworkInfo.getCustomData().get("ApiKey");

                try {
                    AppTracker.startSession(context, apiKey, AppTracker.ENABLE_AUTO_CACHE);
                    listener = new AdListener();
                    AppTracker.setModuleListener(listener);
                    Logger.i(TAG + " Network initialized !");
                    isInitialized = true;
                } catch (Throwable e) {
                    Logger.printStackTrace(TAG + " SDK Init error or SDK error", e);
                }
            } else {
                Logger.i(TAG + " Init Configurations Error !");
            }
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.apptracker.android.track.AppTracker");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }

    @Override
    public void cacheAd(com.pokkt.sdk.AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {

        Logger.i(TAG + " cache Ad called !");

        try {
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                callbacks.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.adConfig == null) {
                this.adConfig = config.clone();
            }

            String adType = "video";
            if (AdFormat.INTERSTITIAL == config.adFormat) {
                adType = "inapp";
            }

            listener.setAdConfig(config);

            if (AppTracker.isAdReady(adType)) {
                callbacks.onSuccess(0.0);
                return;
            }

            this.callback = callbacks;

            isTimedOut = false;
            AppTracker.loadModuleToCache(context, adType);
        } catch (Throwable e) {
            adConfig = null;
            isTimedOut = true;
            Logger.printStackTrace(TAG + e.getMessage(), e);
            callbacks.onFailure("Cache ad Failed");
        }
    }

    @Override
    public void showAd(com.pokkt.sdk.AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.i(TAG + " Show ad called !");

        try {
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                failureCallback.onFailure("Fetch ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.adConfig == null) {
                this.adConfig = config.clone();
            }
            String adType = "video";
            if (AdFormat.INTERSTITIAL == config.adFormat) {
                adType = "inapp";
            }
            if (AppTracker.isAdReady(adType)) {
                AppTracker.loadModule(context, adType);
            } else {
                Logger.i(TAG + " No ad is currently available !");
                requestFailed("Show ad failed", failureCallback);
            }
        } catch (Throwable e) {
            requestFailed("Show ad failed", failureCallback);
        }
    }

    @Override
    public void adClosed(com.pokkt.sdk.AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(com.pokkt.sdk.AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (this.adConfig != null && !(config.equals(this.adConfig))) {
                callbacks.onFailure("Fetch ad Failed. Wrong AdConfig.");
                return;
            }
            if (this.adConfig == null) {
                this.adConfig = config.clone();
            }
            String adType = "video";
            if (AdFormat.INTERSTITIAL == config.adFormat) {
                adType = "inapp";
            }
            if (AppTracker.isAdReady(adType)) {
                callbacks.onSuccess(null);
            } else {
                Logger.i(TAG + " No ad is currently available !");
                callbacks.onFailure("No ad is currently available !");
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        callbacks.onFailure("No ad is currently available !");
    }

    @Override
    public void loadBannerAd(com.pokkt.sdk.AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        callbacks.onFailure("Invalid Network !");
    }

    @Override
    public void destroyBannerAd(com.pokkt.sdk.AdConfig adConfig) {

    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {
        isTimedOut = true;
        String reason = " Time Out In Fetching Ad";
        Logger.i(TAG + reason);
        this.adConfig = null;
    }

    @Override
    public double getAdVC(com.pokkt.sdk.AdConfig adConfig) {
        return 0;
    }

    @Override
    public boolean supportsAdConfig(com.pokkt.sdk.AdConfig config) {
        if (AdFormat.BRANDING == config.adFormat) {
            return false;
        } else if (AdFormat.INTERSTITIAL == config.adFormat && config.isRewarded) {
            return false;
        } else if (AdFormat.VIDEO == config.adFormat && config.isRewarded) {
            return false;
        } else if (AdFormat.BANNER == config.adFormat) {
            return false;
        } else {
            return true;
        }
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(com.pokkt.sdk.AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private void requestFailed(String message, Callbacks.WithOnlyFailure<String> failureCallback) {
        adConfig = null;
        failureCallback.onFailure(message);
    }


    private class AdListener implements AppModuleListener {

        private AdConfig config;

        void setAdConfig(AdConfig c) {
            this.config = c;
        }

        @Override
        public void onModuleCached(String s) {
            Logger.i(TAG + " Ad Received");
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onSuccess(0.0);
            }
        }

        @Override
        public void onModuleLoaded(String s) {
            Logger.i(TAG + " Ad Displayed");
            AdManager.getInstance().adDisplayed(config, adNetworkInfo);
        }

        @Override
        public void onModuleClicked(String s) {

        }

        @Override
        public void onModuleClosed(String s, boolean viewCompleted) {
            Logger.i(TAG + " Ad Closed " + s);
            if (viewCompleted) {
                AdManager.getInstance().adCompleted(config, adNetworkInfo);
            } else {
                AdManager.getInstance().adClosed(config, adNetworkInfo);
            }
            adConfig = null;
        }

        @Override
        public void onModuleFailed(String location, String error, boolean isCache) {
            Logger.i(TAG + " Ad Load Failed " + error);
            if (!isTimedOut) {
                isTimedOut = true;
                callback.onFailure("Ad load failed " + error);
                adConfig = null;
            }
        }
    }
}