package com.pokkt.thirdparty.LoopMe;

import android.app.Activity;
import android.content.Context;

import com.loopme.LoopMeInterstitial;
import com.loopme.common.LoopMeError;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.debugging.Logger;

public class LoopMeInterstitialManager {

    private static final String TAG = LoopMeInterstitialManager.class.getName();
    private final Context context;
    private AdNetworkInfo adNetworkInfo;
    private boolean isTimedOut = false;
    private AdConfig loopmeAdConfig;
    private LoopMeInterstitial interstitial;
    private Callbacks.WithSuccessAndFailure<Double, String> cachingCallback;

    public LoopMeInterstitialManager(AdNetworkInfo info, Context ctx) {
        this.adNetworkInfo = info;
        this.context = ctx;
    }

    public void fetchAd(final AdConfig config, final Callbacks.WithSuccessAndFailure<Double, String> callbacks, final String appKey) {
        Logger.d(TAG + " cache Interstitial called !");

        if (this.loopmeAdConfig != null && !(config.equals(this.loopmeAdConfig))) {
            callbacks.onFailure("Fetch Ad Failed. Wrong AdConfig.");
            return;
        }

        //check if custom data has got the mapping for passed screen name
        final LoopmeAdListener listener = new LoopmeAdListener(config);
        this.cachingCallback = callbacks;

        ((Activity) context).runOnUiThread(new Runnable() {
            @Override
            public void run() {
                try {
                    if (loopmeAdConfig == null) {
                        loopmeAdConfig = config.clone();
                    }
                    interstitial = LoopMeInterstitial.getInstance(appKey, (Activity) context);
                    interstitial.setListener(listener);
                    interstitial.load();
                    isTimedOut = false;
                } catch (Throwable e) {
                    loopmeAdConfig = null;
                    isTimedOut = true;
                    Logger.printStackTrace(TAG + " Cache Ad failed", e);
                    callbacks.onFailure("Caching Failed ");
                }
            }
        });
    }

    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        Logger.d(TAG + " show Interstitial called !");
        try {
            if (this.loopmeAdConfig != null && !(config.equals(this.loopmeAdConfig))) {
                failureCallback.onFailure("Fetch Ad Failed. Wrong AdConfig.");
                return;
            }
            if (interstitial != null && interstitial.isReady()) {
                interstitial.show();
            } else {
                Logger.d(TAG + " No interstitial is currently available !");
                requestFailed("Interstitial Not Available !", failureCallback);
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Show interstitial Failed", e);
            requestFailed("Show interstitial Failed !", failureCallback);
        }
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        loopmeAdConfig = null;
        ((Activity) context).runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (interstitial != null) {
                    interstitial.destroy();
                }
            }
        });
        failureCallback.onFailure(msg);
    }

    public boolean isInterstitialAvailable(AdConfig config) {
        try {
            if (this.loopmeAdConfig != null && !(config.equals(this.loopmeAdConfig))) {
                Logger.d(TAG + " Fetch Ad Failed. Wrong AdConfig.");
                return false;
            }
            if (interstitial != null && interstitial.isReady()) {
                Logger.d(TAG + " Interstitial is available and ready !");
                return true;
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        return false;
    }

    public void notifyCachingTimeout() {
        isTimedOut = true;
        String reason = " Time Out In Fetching interstitial";
        loopmeAdConfig = null;
        ((Activity) context).runOnUiThread(new Runnable() {
            @Override
            public void run() {
                if (interstitial != null) {
                    interstitial.destroy();
                }
            }
        });
        Logger.e(TAG + reason);

    }

    private class LoopmeAdListener implements LoopMeInterstitial.Listener {

        private final AdConfig adConfig;

        LoopmeAdListener(AdConfig config) {
            this.adConfig = config;
        }

        @Override
        public void onLoopMeInterstitialLoadSuccess(LoopMeInterstitial interstitial) {
            Logger.d(TAG + " interstitial Load Success");
            if (!isTimedOut) {
                isTimedOut = true;
                cachingCallback.onSuccess(0.0);
            }
        }

        @Override
        public void onLoopMeInterstitialLoadFail(LoopMeInterstitial loopMeInterstitial, LoopMeError error) {
            Logger.d(TAG + " interstitial Load Fail ! " + error.getMessage());
            if (!isTimedOut) {
                loopmeAdConfig = null;
                ((Activity) context).runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        if (interstitial != null) {
                            interstitial.destroy();
                        }
                    }
                });
                isTimedOut = true;
                cachingCallback.onFailure("Interstitial Load Failed ! " + error.getMessage());
            }
        }

        @Override
        public void onLoopMeInterstitialShow(LoopMeInterstitial interstitial) {
            Logger.d(TAG + " Interstitial Show");
            AdManager.getInstance().adDisplayed(adConfig, adNetworkInfo);
        }

        @Override
        public void onLoopMeInterstitialHide(LoopMeInterstitial interstitial) {
            Logger.d(TAG + " Interstitial Hide");
            AdManager.getInstance().adClosed(adConfig, adNetworkInfo);
            loopmeAdConfig = null;
            if (interstitial != null) {
                interstitial.destroy();
            }
        }

        @Override
        public void onLoopMeInterstitialClicked(LoopMeInterstitial interstitial) {

        }

        @Override
        public void onLoopMeInterstitialLeaveApp(LoopMeInterstitial interstitial) {
            Logger.d(TAG + " Interstitial LeaveApp");
            AdManager.getInstance().adSkipped(adConfig, adNetworkInfo);
            loopmeAdConfig = null;
        }

        @Override
        public void onLoopMeInterstitialExpired(LoopMeInterstitial interstitial) {
            Logger.d(TAG + " Interstitial Expired");
            if (interstitial != null) {
                interstitial.load();
            }
        }

        @Override
        public void onLoopMeInterstitialVideoDidReachEnd(LoopMeInterstitial interstitial) {
            Logger.d(TAG + " Interstitial VideoDidReachEnd");
            AdManager.getInstance().adCompleted(adConfig, adNetworkInfo);
            if (adConfig.isRewarded) {
                AdManager.getInstance().adGratified(adConfig, 0, adNetworkInfo);
            }
        }
    }
}