package com.pokkt.thirdparty;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.Keep;

import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.pokkt.thirdparty.RevMob.RevMobBannerManager;
import com.pokkt.thirdparty.RevMob.RevMobInterstitialManager;
import com.pokkt.thirdparty.RevMob.RevMobNonRewardedVideoManager;
import com.pokkt.thirdparty.RevMob.RevMobRewardedVideoManager;

import rm.com.android.sdk.Rm;

@Keep
public class RevMobNetwork implements AdNetwork {

    private static final String TAG = RevMobNetwork.class.getName();

    private Context context = null;
    private AdNetworkInfo adNetworkInfo;
    private boolean isInitialized = false;
    private RevMobInterstitialManager interstitialManager;
    private RevMobNonRewardedVideoManager nonRewardedVideoManager;
    private RevMobRewardedVideoManager rewardedVideoManager;
    private RevMobBannerManager bannerManager;
    private static final String APP_ID = "appId";

    @Override
    public void initNetwork(final Context context, final AdNetworkInfo info) {
        Logger.d(TAG+ " init network called !");

        synchronized (RevMobNetwork.class) {
            if (isInitialized) {
                Logger.d(TAG+ " Network already initialized !");
                return;
            }
            if (!(context instanceof Activity)) {
                Logger.d(TAG+ " Activity context needed !");
                return;
            }
            this.context = context;

            if (!sdkEnabled()) {
                Logger.d(TAG+ " Init Configurations Error !");
                return;
            }

            try {
                if (extrasAreValid(info)) {
                    this.adNetworkInfo = info;
                    String appId = info.getCustomData().get(APP_ID);
                    Rm.init(context, appId);
                    isInitialized = true;
                    //58c2609c1d1f4e1a0d3895ae
                    interstitialManager = new RevMobInterstitialManager(info);
                    nonRewardedVideoManager = new RevMobNonRewardedVideoManager(info);
                    rewardedVideoManager = new RevMobRewardedVideoManager(info);
                    bannerManager = new RevMobBannerManager(info);
                } else {
                    Logger.d(TAG+ " Init Configurations Error or SDK Error!");
                }
            } catch (Throwable e) {
                Logger.d(TAG+ " Init Configurations Error or SDK Error !");
            }
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    private boolean extrasAreValid(AdNetworkInfo info) {
        return info != null && info.getCustomData() != null && info.getCustomData().containsKey(APP_ID);
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        if (AdFormat.VIDEO == config.adFormat) {
            if (config.isRewarded) {
                rewardedVideoManager.fetchAd(config, callbacks);
            } else {
                nonRewardedVideoManager.fetchAd(config, callbacks);
            }
        } else if (AdFormat.INTERSTITIAL == config.adFormat) {
            interstitialManager.fetchAd(config, callbacks);
        } else {
            callbacks.onFailure("Invalid Ad Format");
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {
        if (AdFormat.VIDEO == config.adFormat) {
            if (config.isRewarded) {
                rewardedVideoManager.showAd(config, failureCallback, context);
            } else {
                nonRewardedVideoManager.showAd(config, failureCallback, context);
            }
        } else if (AdFormat.INTERSTITIAL == config.adFormat) {
            interstitialManager.showAd(config, failureCallback, context);
        } else {
            failureCallback.onFailure("Invalid Ad Format");
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {

    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (AdFormat.VIDEO == config.adFormat) {
                if (config.isRewarded) {
                    if (rewardedVideoManager.isVideoAvailable(config)) {
                        callbacks.onSuccess(null);
                        return;
                    }
                } else {
                    if (nonRewardedVideoManager.isVideoAvailable(config)) {
                        callbacks.onSuccess(null);
                        return;
                    }
                }
            }
            if (AdFormat.INTERSTITIAL == config.adFormat) {
                if (interstitialManager.isInterstitialAvailable(config)) {
                    callbacks.onSuccess(null);
                    return;
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG+ " Failed to find availability", e);
        }
        callbacks.onFailure("Ad Not Available");
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        bannerManager.loadBanner(adConfig, bannerUnit, callbacks, (Activity) context);
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {
        bannerManager.destroyBanner();
    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {
        if (AdFormat.VIDEO == adConfig.adFormat) {
            if (adConfig.isRewarded) {
                rewardedVideoManager.notifyCachingTimeout();
            } else {
                nonRewardedVideoManager.notifyCachingTimeout();
            }
        } else if (AdFormat.INTERSTITIAL == adConfig.adFormat) {
            interstitialManager.notifyCachingTimeout();
        } else if (AdFormat.BANNER == adConfig.adFormat) {
            bannerManager.notifyCachingTimeout();
        }
    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        return 0.0;
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        if (AdFormat.BRANDING == config.adFormat) {
            return false;
        }
        if (AdFormat.INTERSTITIAL == config.adFormat && config.isRewarded) {
            return false;
        }

        return true;
    }

    @Override
    public boolean isInitialised() {
        return isInitialized;
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("rm.com.android.sdk.Rm");
            Class.forName("rm.com.android.sdk.RmListener");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG+ " SDK not found");
            return false;
        }
    }
}