package com.pokkt.thirdparty;

import android.app.Activity;
import android.content.Context;
import android.support.annotation.Keep;
import android.text.TextUtils;

import com.pokkt.PokktAds;
import com.pokkt.sdk.AdConfig;
import com.pokkt.sdk.AdManager;
import com.pokkt.sdk.Callbacks;
import com.pokkt.sdk.adnetworks.AdFormat;
import com.pokkt.sdk.adnetworks.AdNetwork;
import com.pokkt.sdk.adnetworks.AdNetworkInfo;
import com.pokkt.sdk.banners.BannerUnit;
import com.pokkt.sdk.debugging.Logger;
import com.pokkt.sdk.models.adcampaign.AdCampaign;
import com.unity3d.ads.IUnityAdsListener;
import com.unity3d.ads.UnityAds;
import com.unity3d.ads.UnityAds.FinishState;
import com.unity3d.ads.UnityAds.UnityAdsError;
import com.unity3d.ads.properties.SdkProperties;

import org.json.JSONObject;

@Keep
public class Unity3DAdNetwork implements AdNetwork {
    private static final String TAG = Unity3DAdNetwork.class.getName();

    private static final String GAME_ID = "gameId";
    private static final String REWARD_AMOUNT = "rewardAmount";

    private Activity activity;
    private AdConfig unityAdConfig;
    private AdNetworkInfo adNetworkInfo;
    private UnityListener listener;
    private double rewardValue = 0;

    @Override
    public void initNetwork(Context context, AdNetworkInfo info) {
        synchronized (Unity3DAdNetwork.class) {

            Logger.i(TAG + " Minimum Required Unity-Ads Version is 2.0, found: " + UnityAds.getVersion());

            if (UnityAds.isInitialized()) {
                Logger.i(TAG + " Network already initialized !");
                return;
            }

            if (!(context instanceof Activity)) {
                Logger.e(TAG + " Init Configurations Error. Needs Activity context");
                return;
            }

            if (!sdkEnabled()) {
                return;
            }

            this.activity = (Activity) context;
            this.adNetworkInfo = info;

            try {
                String gameId = "";
                if (extrasAreValid()) {
                    gameId = adNetworkInfo.getCustomData().get(GAME_ID);
                    if (adNetworkInfo.getCustomData().containsKey(REWARD_AMOUNT)) {
                        rewardValue = Double.parseDouble(adNetworkInfo.getCustomData().get(REWARD_AMOUNT));
                    }

                    Logger.i(TAG + " Received data from server are GameId: " + gameId + ", RewardAmount:" + rewardValue);
                } else {
                    throw new Exception("Unity3DAdNetwork Init Configurations Error!");
                }

                boolean testMode = PokktAds.Debugging.isEnabled();
                boolean debugMode = PokktAds.Debugging.isEnabled();

//                MediationMetaData mediationMetaData = new MediationMetaData(activity);
//                mediationMetaData.setName("mediationPartner");
//                mediationMetaData.setVersion("v12345");
//                mediationMetaData.commit();
//
//                MetaData debugMetaData = new MetaData(activity);
//                debugMetaData.set("test.debugOverlayEnabled", true);
//                debugMetaData.commit();

                UnityAds.setDebugMode(debugMode);
                listener = new UnityListener();
                UnityAds.initialize(activity, gameId, listener, testMode);
                UnityAds.setListener(listener);
            } catch (Throwable e) {
                Logger.printStackTrace(TAG + " Init Configurations Error or SDK Error!", e);
            }
        }
    }

    @Override
    public AdNetworkInfo getAdNetworkInfo() {
        return adNetworkInfo;
    }

    private boolean sdkEnabled() {
        try {
            Class.forName("com.unity3d.ads.UnityAds");
            return true;
        } catch (Throwable ignore) {
            Logger.e(TAG + " SDK not found");
            return false;
        }
    }

    private boolean extrasAreValid() {
        return adNetworkInfo != null
                && adNetworkInfo.getCustomData() != null
                && adNetworkInfo.getCustomData().containsKey(GAME_ID);
    }

    @Override
    public void cacheAd(AdConfig config, Callbacks.WithSuccessAndFailure<Double, String> callbacks) {
        try {
            Logger.i(TAG + " Cache Ad called !");
            if (unityAdConfig != null && !unityAdConfig.equals(config)) {
                callbacks.onFailure("Cache Ad Failed !");
                return;
            }
            if (unityAdConfig == null) {
                unityAdConfig = config.clone();
            }
            String placementId = getPlacementId(config);
            if (SdkProperties.isInitialized() && UnityAds.isReady(placementId)) {
                rewardValue = config.isRewarded ? rewardValue : 0;
                callbacks.onSuccess(0.0);
            } else {
                callbacks.onFailure("Ad Caching Failed");
                unityAdConfig = null;
            }

        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Cache Ad failed ", e);
            unityAdConfig = null;
            callbacks.onFailure("Ad Caching Failed");
        }
    }

    @Override
    public void showAd(AdConfig config, Callbacks.WithOnlyFailure<String> failureCallback) {

        try {
            Logger.i(TAG + " Play Ad called !");
            if (unityAdConfig == null || unityAdConfig.equals(config)) {
                if (unityAdConfig == null) {
                    unityAdConfig = config.clone();
                }
                String placementId = getPlacementId(config);
                if (UnityAds.isReady(placementId)) {
                    listener.setConfig(unityAdConfig);
//                    PlayerMetaData playerMetaData = new PlayerMetaData(activity);
//                    playerMetaData.setServerId("rikshot");
//                    playerMetaData.commit();
//
//                    MediationMetaData ordinalMetaData = new MediationMetaData(activity);
//                    ordinalMetaData.setOrdinal(ordinal++);
//                    ordinalMetaData.commit();
                    UnityAds.show(activity, placementId);
                } else {
                    Logger.i(TAG + " No Rewarded Ad is currently available !");
                    requestFailed("Ad Not Available !", failureCallback);
                }
            } else {
                requestFailed("Invalid AdConfig", failureCallback);
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Play Ad Failed", e);
            requestFailed("Play Ad Failed !", failureCallback);
        }
    }

    @Override
    public void adClosed(AdConfig adConfig) {
    }

    @Override
    public void checkAdAvailability(AdConfig config, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        try {
            if (unityAdConfig == null || unityAdConfig.equals(config)) {
                String placementId = getPlacementId(config);
                if (UnityAds.isReady(placementId)) {
                    callbacks.onSuccess(null);
                    return;
                }
            }
        } catch (Throwable e) {
            Logger.printStackTrace(TAG + " Failed to find availability", e);
        }
        callbacks.onFailure("Ad not available");
    }

    @Override
    public void loadBannerAd(AdConfig adConfig, BannerUnit bannerUnit, Callbacks.WithSuccessAndFailure<AdCampaign, String> callbacks) {
        callbacks.onFailure("Invalid Network !");
    }

    @Override
    public void destroyBannerAd(AdConfig adConfig) {

    }

    @Override
    public void notifyCachingTimeout(AdConfig adConfig) {

    }

    @Override
    public double getAdVC(AdConfig adConfig) {
        if (adConfig.isRewarded) {
            return rewardValue;
        } else {
            return 0;
        }
    }

    @Override
    public boolean supportsAdConfig(AdConfig config) {
        return AdFormat.VIDEO == config.adFormat;
    }

    @Override
    public boolean isInitialised() {
        return UnityAds.isInitialized();
    }

    @Override
    public boolean isPokktNetwork() {
        return false;
    }

    @Override
    public boolean verifyCampaignForAdConfig(AdConfig adConfig, boolean verifyCached) {
        return true;
    }

    private String getPlacementId(AdConfig config) {
        String placementId = "";
        if (adNetworkInfo.getCustomData().containsKey("screens_mapping_data")) {
            try {
                JSONObject mappingObject = new JSONObject(adNetworkInfo.getCustomData().get("screens_mapping_data"));
                String mapData = mappingObject.optString(config.screenName);
                if (!TextUtils.isEmpty(mapData)) {
                    JSONObject mapObj = new JSONObject(mapData);
                    int isRewarded = mapObj.optInt("incent");
                    if (config.isRewarded) {
                        if (isRewarded == 1) {
                            placementId = mapObj.optString("network_screen", "").trim();
                        } else {
                            Logger.e(TAG + " Failed. Dashboard setting and adconfig request not matching.");
                            return placementId;
                        }
                    } else {
                        if (isRewarded == 0) {
                            placementId = mapObj.optString("network_screen", "").trim();
                        } else {
                            Logger.e(TAG + " Failed. Dashboard setting and adconfig request not matching.");
                            return placementId;
                        }
                    }
                }
            } catch (Exception e) {
                Logger.printStackTrace(TAG + " Invalid Mapping Json", e);
            }
        }
        if (placementId.length() == 0) {
            placementId = config.screenName;
        }
        return placementId;
    }

    private void requestFailed(String msg, Callbacks.WithOnlyFailure<String> failureCallback) {
        unityAdConfig = null;
        failureCallback.onFailure(msg);
    }

    private class UnityListener implements IUnityAdsListener {

        private AdConfig config;

        public void setConfig(com.pokkt.sdk.AdConfig config) {
            this.config = config;
        }

        @Override
        public void onUnityAdsError(UnityAdsError unityError, String message) {
            Logger.i(TAG + unityError.name() + ": " + message);
        }

        @Override
        public void onUnityAdsFinish(String placementId, FinishState finishState) {
            Logger.i(TAG + " Ad Finished, state is: " + finishState.name());
            unityAdConfig = null;
            if (finishState == FinishState.SKIPPED || finishState == FinishState.ERROR) {
                AdManager.getInstance().adSkipped(config, adNetworkInfo);
            } else {
                AdManager.getInstance().adCompleted(config, adNetworkInfo);
                AdManager.getInstance().adClosed(config, adNetworkInfo);
                if (config.isRewarded) {
                    AdManager.getInstance().adGratified(config, rewardValue, adNetworkInfo);
                }
            }
        }

        @Override
        public void onUnityAdsReady(String placementId) {
            Logger.i(TAG + " Ad is Ready for placement id: " + placementId);
        }

        @Override
        public void onUnityAdsStart(String placementId) {
            Logger.i(TAG + " Ad is Ready for placement id: " + placementId);
            AdManager.getInstance().adDisplayed(config, adNetworkInfo);
        }
    }
}
